from collections import Counter
from functools import lru_cache

from .my_exceptions import UniqueTypeError


@lru_cache(maxsize=None)
def unique_symbol(line: str) -> int:
    if not isinstance(line, str):
        raise UniqueTypeError('Type does not match')
    return list(Counter(line).values()).count(1)


def unique_symbol_in_list(line: list) -> list:
    if not isinstance(line, list):
        raise UniqueTypeError('Data type must be list')
    return list(map(unique_symbol, line))



import argparse
from argparse import Namespace

from collection_framework.unique_collection import unique_symbol

from .my_exceptions import NoArgumentsError


def get_args() -> Namespace:
    parser = argparse.ArgumentParser(description='Counting unique symbols')
    parser.add_argument("--string", "-s", help='Enter data for processing')
    parser.add_argument("--file",
                        "-f",
                        type=argparse.FileType(),
                        help='Enter file name')
    return parser.parse_args()


def get_result() -> int:
    args = get_args()
    if not args.file and not args.string:
        raise NoArgumentsError('You need to enter arguments')
    if args.file:
        return unique_symbol(args.file.read())
    return unique_symbol(args.string)



import inspect

import pytest

from collection_framework.unique_collection import (UniqueTypeError,
                                                    unique_symbol,
                                                    unique_symbol_in_list)


@pytest.mark.parametrize('test_input', [1, 1.1, {'qe': 'eq'}])
def test_type_one(test_input):
    with pytest.raises(UniqueTypeError):
        inspect.unwrap(unique_symbol)(test_input)


@pytest.mark.parametrize('test_input, expected', [('abbbccdf', 3),
                                                  ('adfgfadd', 1), ('', 0),
                                                  ('asdfghj', 7)])
def test_unique_symbol_one(test_input, expected):
    assert unique_symbol(test_input) == expected


@pytest.mark.parametrize('test_input, expected', [(['abc123'], [6]),
                                                  (['aaabbbccc123'], [3]),
                                                  ([''], [0])])
def test_unique_symbol_in_list(test_input, expected):
    assert unique_symbol_in_list(test_input) == expected


@pytest.mark.parametrize('test_input', [1, 1.1, {'qe': 'fa'}])
def test_type_in_list(test_input):
    with pytest.raises(UniqueTypeError):
        unique_symbol_in_list(test_input)



from argparse import Namespace
from unittest.mock import mock_open, patch

import pytest

from collection_framework import NoArgumentsError, get_result


@patch('collection_framework.collect_framework.unique_symbol')
@patch('collection_framework.collect_framework.argparse.ArgumentParser.parse_args')
def test_string(mock, mock_unique_symbol):
    mock.return_value = Namespace(file=None, string='assdqrt')
    mock_unique_symbol.return_value = 5
    assert get_result() == 5
    mock_unique_symbol.assert_called_once_with('assdqrt')


@patch('collection_framework.collect_framework.unique_symbol')
@patch('collection_framework.collect_framework.argparse.ArgumentParser.parse_args')
@patch('tests.test_collect_framework.open', mock_open(read_data='asdas\nas'))
def test_file(mock, mock_unique_symbol):
    mock.return_value = Namespace(file=open(), string=None)
    mock_unique_symbol.return_value = 2
    assert get_result() == 2
    mock_unique_symbol.assert_called_once_with('asdas\nas')


@patch('collection_framework.collect_framework.unique_symbol')
@patch('collection_framework.collect_framework.argparse.ArgumentParser.parse_args')
@patch('tests.test_collect_framework.open', mock_open(read_data='asdas\nas'))
def test_file_priority(mock, mock_unique_symbol):
    mock.return_value = Namespace(file=open(), string='qwerasdasd')
    mock_unique_symbol.return_value = 2
    assert get_result() == 2
    mock_unique_symbol.assert_called_once_with('asdas\nas')


@patch('collection_framework.collect_framework.argparse.ArgumentParser.parse_args')
def test_exeption(mock):
    mock.return_value = Namespace(file=None, string=None)
    with pytest.raises(NoArgumentsError) as excinfo:
        get_result()
    assert str(excinfo.value) == 'You need to enter arguments'
